/* Copyright (c) 2001 Bjrn Johannesson */
/*
 * Small biff app for larswm. It updates the status bar by setting
 * the LARSWM_BARTEXT atom every second.
 * A quick and dirty hack of the larsclock program by Lars Bernhardsson.
 * When you get mail you will hear 3 beeps and the bar will say 
 * "### New Mail! ###" when there is no mail it will show the time.
 * I wrote this because my sh-script was too cpu-intensive.
 * This is very basic. If it doesn't work for you: change it
 * or just forget about it...
 * Check larsclock.c for original source (in the larswm-7.0.13 package)
 * See the README file for License details
 */

#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>  /* For the stat() function */
#include <limits.h>
#include <pwd.h>

#include <X11/X.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xresource.h>

#include "config.h"
#include "larsbiff.h"

int main(int argc, char **argv)
{
	time_t t = 0;
	size_t len;
	int i;
	int retval;
	int bell = 0;
	int have_mail = 0;
	int swap_logo = 0;
	int seconds = SECONDS;
	double load[3];
	char bar_text[BAR_TEXT_LENGTH+1] = "";
	char *prepend_str = NULL;
	char *slist;
	char *display_string = NULL;
	char rcfile[PATH_MAX+1] = "";
	const char *msg[] = { 	"### New Mail! ###",
				"-## New Mail! ##-",
				"--# New Mail! #--",
				"--- New Mail! ---",
				"--# New Mail! #--",
				"-## New Mail! ##-" };
	struct passwd *pass;
	Display *dpy;
	Atom bartext_larswm;
	XTextProperty pr;

	for (i = 1; i < argc; i++)
	{
		if (!strcmp(argv[i], "-display") && i+1<argc) {
			display_string = argv[++i];
		}
		else if (!strcmp(argv[i], "-v")) {
			print_version();
			exit(EXIT_SUCCESS);
		}
		else if (!strcmp(argv[i], "-h")) {
			usage();
			exit(EXIT_SUCCESS);
		}
		else if (!strcmp(argv[i], "-string") && i+1<argc) {
			prepend_str = strdup(argv[++i]);
		}
		else if (!strcmp(argv[i], "-defaults")) {
			print_prefs();
			exit(EXIT_SUCCESS);
		}
		else if (!strcmp(argv[i], "-f") && i+1<argc) {
			strncpy(rcfile, argv[++i], PATH_MAX);
		}
		else {
			fprintf(stderr, "larsbiff -h for help\n");
			exit(EXIT_FAILURE);
		}
	}
	
	dpy = XOpenDisplay(display_string);
	if (dpy == 0)
	{
		fprintf(stderr, "Unable to open display\n");
		exit(EXIT_FAILURE);
	}
	
	if (!strlen(rcfile)) {
		pass = getpwuid(getuid());
		if (pass == NULL) {
			fprintf(stderr, "You don't exist, go away!\n");
			exit(EXIT_FAILURE);
		}
		else {
			retval = snprintf(rcfile, PATH_MAX, "%s%s",
				pass->pw_dir, "/.larsbiffrc");
			if (retval > PATH_MAX) {
				fprintf(stderr, "rc-path to long!\n");
				exit(EXIT_FAILURE);
			}
		}
	}

	XrmInitialize();
	load_config(rcfile);
	
	bartext_larswm = XInternAtom(dpy, "LARSWM_BARTEXT", False);
	
	/* Removed because larswm may not be running yet
	bartext_larswm = XInternAtom(dpy, "LARSWM_BARTEXT", True);
	if (!bartext_larswm) { 
		fprintf(stderr, "You do not seem to be running larswm\n");
		exit(EXIT_FAILURE);
	}
	*/ 
	i = prefs.maildelay;
	bell = prefs.bells;

	for ( ;; )
	{
		if (i == prefs.maildelay)
			have_mail = checkmail();

		if (!have_mail) {
			if (prepend_str != NULL) {
				strncpy(bar_text, prepend_str, BAR_TEXT_LENGTH-3);
				strcat(bar_text, "  ");
			}
			if (prefs.show_load) {
			getloadavg(load, 3);
			len = strlen(bar_text);
			snprintf(bar_text+len,BAR_TEXT_LENGTH-len-1, "[%.2f] ", load[0]);
			}
			time(&t);
			len = strlen(bar_text);
			strftime(bar_text+len,
					BAR_TEXT_LENGTH-len-1,
					prefs.timeformat, localtime(&t));
			bell = prefs.bells;
			swap_logo = 0;
		} else {
			if (bell) {
				XBell(dpy, 100);
				bell--;
			}
			strcpy(bar_text, msg[swap_logo]);
			swap_logo = (swap_logo + 1) % 6;
		}
		if (i == seconds)
			i = 0;
		
		slist = (char *)bar_text;
		XStringListToTextProperty(&slist, 1, &pr);
		XSetTextProperty(dpy, DefaultRootWindow(dpy), &pr, bartext_larswm);
		XFree(pr.value);
		XSync(dpy, False);

		sleep(1);
		i++;
		bar_text[0] = '\0';
	}
}

/* Check mail in one mailbox */
int checkone(char *f)
{
	/* We do not need to create mbox_stat every time */
	static struct stat mbox_stat;
	if ( stat(f, &mbox_stat) <  0) {
		fprintf(stderr, "Unable to stat %s exiting\n", f);
		exit(EXIT_FAILURE);
	}
	if (mbox_stat.st_size > 0 && (mbox_stat.st_mtime >= mbox_stat.st_atime)) {
		return 1;
	}
	return 0;
}

/* Check mail in all mailboxes */
int checkmail()
{
	static int i;

	for (i = 0; i < MAX_MAILBOXES; i++) {
		if (prefs.mailbox[i] && checkone(prefs.mailbox[i]))
			return 1;
	}
	return 0;
}

/* Initialize the preferenses to default values */
void init_data()
{
	int i;
	memset(&prefs, 0, sizeof(prefs));
	
	prefs.bells = BELLS;
	prefs.maildelay = SECONDS;
	prefs.show_load = 0;
	prefs.timeformat = TIME_FORMAT;
	for (i = 0; i < MAX_MAILBOXES; i++)
		prefs.mailbox[i] = NULL;
}

/* Parse the configfile */
void load_config(char *file)
{
	XrmDatabase data = (XrmDatabase)0;
	XrmValue val;
	char *vt;
	char pname[32];
	char pclass[32];
	int i;
	int tmp;
	
	init_data();

	if ((data = XrmGetFileDatabase(file)) == NULL) {
		return;
	}

	if (XrmGetResource(data, "larsbiff.bells", "Larsbiff.Bells", &vt, &val))
		prefs.bells = atoi(val.addr);

	if (XrmGetResource(data, "larsbiff.maildelay", "Larsbiff.Maildelay", &vt, &val)) {
		tmp = atoi(val.addr);
		if (tmp != 0)
			prefs.maildelay = tmp;
	}

	if (XrmGetResource(data, "larsbiff.show_load", "Larsbiff.Show_load", &vt, &val))
		prefs.show_load = get_bool(val.addr);

	if(XrmGetResource(data, "larsbiff.timeformat", "Larsbiff.Timeformat", &vt, &val))
		prefs.timeformat = strdup(val.addr);

	for (i = 0; i < MAX_MAILBOXES; i++) {
		sprintf(pname, "larsbiff.mailbox.%d", i);
		sprintf(pclass, "Larsbiff.Mailbox.%d", i);
		if (XrmGetResource(data, pname, pclass, &vt, &val)) {
			prefs.mailbox[i] = strdup(val.addr);
		}
	}
	
}

void usage()
{
	puts("usage: larsbiff [options]\nOptions:");
	puts(" -h                           Print this help");
	puts(" -v                           Print version");
	puts(" -string str                  Show str to the left of the time");
	puts(" -display display             Open on which display");
	puts(" -f file                      Use file instead of $HOME/.larsbiffrc");
	
}

void print_version()
{
	puts(LARSBIFF_VERSION);
}

int get_bool(char *str)
{
	if (!strcasecmp("true", str))
		return 1;
	return 0;
}

void print_prefs()
{
	puts("!!!!!!!!!!!!!!!!!!!!!!!!!!");
	puts("! larsbiff configuration !");
	puts("!!!!!!!!!!!!!!!!!!!!!!!!!!");
	puts("!");
	printf("! %s\n", LARSBIFF_VERSION);
	puts("!");
	puts("! Don\'t forget to uncomment!");
	puts("!");
	puts("! How many beeps when you get mail");
	puts("!");
	puts("! larsbiff.bells: 3");
	puts("!");
	puts("! How often do you want to check for mail");
	puts("!");
	puts("! larsbiff.maildelay: 10");
	puts("!");
	puts("! Show loadavg on the status bar");
	puts("!");
	puts("! larsbiff.show_load: True");
	puts("!");
	puts("! Path to mailboxes");
	puts("! Don\'t forget to set the right MAX_MAILBOXES in config.h");
	puts("! Default is 10");
	puts("!");
	puts("! larsbiff.mailbox.0: /home/bjorn/mail/mbox");
	puts("! larsbiff.mailbox.1: /home/bjorn/mail/larswm");
	puts("! larsbiff.mailbox.2: /home/bjorn/mail/cc65");
	puts("!");
	puts("! End of configuration");
}

